﻿using System;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using Microsoft.IdentityModel.Claims;
using Microsoft.IdentityModel.Configuration;
using Microsoft.IdentityModel.Protocols.WSTrust;
using Microsoft.IdentityModel.SecurityTokenService;


namespace BasicSTS
{

    class CustomSecurityTokenService : SecurityTokenService
    {
        string _addressExpected = "http://some";
        public CustomSecurityTokenService(SecurityTokenServiceConfiguration config)
            : base(config)
        {
        }

        protected override Scope GetScope(IClaimsPrincipal principal, RequestSecurityToken request)
        {
            var appliesTo = request.AppliesTo.Uri.AbsoluteUri;
            Scope scope = new Scope(appliesTo,
                SecurityTokenServiceConfiguration.SigningCredentials);

            scope.ReplyToAddress = request.ReplyTo;
            if (string.IsNullOrEmpty(scope.ReplyToAddress))
                scope.ReplyToAddress = request.AppliesTo.Uri.AbsoluteUri;

            //Public key from this cert will be used to encrypt the token.
            //only public key cert. use to secure communication.
            scope.EncryptingCredentials = new X509EncryptingCredentials(GetRPCert(request.AppliesTo));

            return scope;
        }

        private static X509Certificate2 GetRPCert(EndpointAddress appliesTo)
        {
            if (appliesTo.Uri.AbsoluteUri.EndsWith("WFFacade"))
                return new X509Certificate2(@"WFServiceFacade.cer");

            return new X509Certificate2(@"SampleService.cer");
        }

        void ValidateAppliesTo(EndpointAddress appliesTo)
        {
            if (appliesTo == null)
            {
                throw new InvalidRequestException("The appliesTo is null.");
            }

            if (!appliesTo.Uri.Equals(new Uri(_addressExpected)))
            {
                throw new InvalidRequestException(String.Format("The relying party address is not valid. Expected value is {0}, the actual value is {1}.", _addressExpected, appliesTo.Uri.AbsoluteUri));
            }
        }

        public override RequestSecurityTokenResponse Issue(IClaimsPrincipal principal, RequestSecurityToken request)
        {
            Console.WriteLine("Issuing token...");
            var token = base.Issue(principal, request);
            return token;
        }


        protected override IClaimsIdentity GetOutputClaimsIdentity(IClaimsPrincipal principal, RequestSecurityToken request, Scope scope)
        {
            // ValidateAppliesTo(request.AppliesTo);

            // Create new identity and copy content of the caller's identity into it (including the existing delegate chain)
            IClaimsIdentity callerIdentity = (IClaimsIdentity)principal.Identity;
            IClaimsIdentity outputIdentity = callerIdentity.Copy();


            // If there is an ActAs token in the RST, return a copy of it as the top-most identity
            // and put the caller's identity into the Actor property of this identity.
            if (request.ActAs != null)
            {
                IClaimsIdentity actAsSubject = request.ActAs.GetSubject()[0];
                IClaimsIdentity actAsIdentity = actAsSubject.Copy();

                // Find the last actor in the actAs identity
                IClaimsIdentity lastActor = actAsIdentity;
                while (lastActor.Actor != null)
                {
                    lastActor = lastActor.Actor;
                }

                // Set the caller's identity as the last actor in the delegation chain
                lastActor.Actor = outputIdentity;

                // Return the actAsIdentity instead of the caller's identity in this case
                outputIdentity = actAsIdentity;
            }
            if (outputIdentity.Claims.Count == 0) //add hardcoded claims...
                outputIdentity.Claims.AddRange(new Claim[]
                {
                    new Claim(System.IdentityModel.Claims.ClaimTypes.Name, "Zulfiqar"),
                    new Claim(System.IdentityModel.Claims.ClaimTypes.Email, "me@zamd.net"),
                    new Claim(System.IdentityModel.Claims.ClaimTypes.Upn, "zamd@zamd.net"),
                });
            return outputIdentity;
        }
    }
}